@@ -33,6 +33,7 @@ from kikit.sexpr import isElement, parseSexprF, SExpr, Atom, findNode, parseSexp
 from kikit.annotations import AnnotationReader, TabAnnotation
 from kikit.drc import DrcExclusion, readBoardDrcExclusions, serializeExclusion
 from kikit.units import mm, deg
+from kikit.pcbnew_utils import increaseZonePriorities
 
 class PanelError(RuntimeError):
     pass
@@ -55,11 +56,11 @@ class GridPlacerBase:
         """
         raise NotImplementedError("GridPlacerBase.position has to be overridden")
 
-    def rotation(self, i: int, j: int) -> int:
+    def rotation(self, i: int, j: int) -> KiAngle:
         """
         Given row and col coords of a board, return the orientation of the board
         """
-        return 0
+        return EDA_ANGLE(0, pcbnew.DEGREES_T)
 
 class BasicGridPosition(GridPlacerBase):
     """
@@ -92,8 +93,6 @@ class BasicGridPosition(GridPlacerBase):
                hbonecount * (self.hbonewidth + self.verSpace)
         return toKiCADPoint((xPos, yPos))
 
-    def rotation(self, i: int, j: int) -> KiAngle:
-        return EDA_ANGLE(0, pcbnew.DEGREES_T)
 
 class OddEvenRowsPosition(BasicGridPosition):
     """
@@ -119,7 +118,7 @@ class OddEvenRowsColumnsPosition(BasicGridPosition):
     """
     def rotation(self, i: int, j: int) -> KiAngle:
         if (i % 2) == (j % 2):
-            return 0
+            return EDA_ANGLE(0, pcbnew.DEGREES_T)
         return EDA_ANGLE(180, pcbnew.DEGREES_T)
 
 
@@ -338,10 +337,6 @@ def isBoardEdge(edge):
     """
     return isinstance(edge, pcbnew.PCB_SHAPE) and edge.GetLayerName() == "Edge.Cuts"
 
-def increaseZonePriorities(board, amount=1):
-    for zone in board.Zones():
-        zone.SetAssignedPriority(zone.GetAssignedPriority() + amount)
-
 def tabSpacing(width, count):
     """
     Given a width of board edge and tab count, return an iterable with tab
@@ -436,7 +431,10 @@ def bakeTextVars(board: pcbnew.BOARD) -> None:
     for drawing in board.GetDrawings():
         if not isinstance(drawing, pcbnew.PCB_TEXT):
             continue
-        drawing.SetText(drawing.GetShownText())
+        if isV8():
+            drawing.SetText(drawing.GetShownText(True))
+        else:
+            drawing.SetText(drawing.GetShownText())
 
 class Panel:
     """
@@ -618,7 +616,7 @@ class Panel:
             pass
 
     def writeCustomDrcRules(self):
-        with open(self.getDruFilepath(), "w", encoding="utf-8") as f:
+        with open(self.getDruFilepath(), "w+", encoding="utf-8") as f:
             f.write("(version 1)\n\n")
             for r in self.customDRCRules:
                 f.write(str(r))
@@ -1006,10 +1004,11 @@ class Panel:
             # the attribute must be first removed without changing the
             # orientation of the text.
             for item in (*footprint.GraphicalItems(), footprint.Value(), footprint.Reference()):
-                if isinstance(item, pcbnew.FP_TEXT) and item.IsKeepUpright():
+                if isinstance(item, pcbnew.FIELD_TYPE) and item.IsKeepUpright():
                     actualOrientation = item.GetDrawRotation()
                     item.SetKeepUpright(False)
-                    item.SetTextAngle(actualOrientation - footprint.GetOrientation())
+                    alteredOrientation = item.GetDrawRotation()
+                    item.SetTextAngle(item.GetTextAngle() + (alteredOrientation - actualOrientation))
             footprint.Rotate(originPoint, rotationAngle)
             footprint.Move(translation)
             edges += removeCutsFromFootprint(footprint)
@@ -1372,7 +1371,7 @@ class Panel:
         boardSlot = GeometryCollection()
         for s in self.substrates:
             boardSlot = boardSlot.union(s.exterior())
-        boardSlot = boardSlot.buffer(slotwidth)
+        boardSlot = boardSlot.buffer(slotwidth, join_style="mitre")
         frameBody = box(*self.boardSubstrate.bounds()).difference(boardSlot)
         self.appendSubstrate(frameBody)
 
@@ -1447,7 +1446,12 @@ class Panel:
         """
         for cut in cuts:
             if len(cut.simplify(SHP_EPSILON).coords) > 2 and not boundCurves:
-                raise RuntimeError("Cannot V-Cut a curve")
+                message = "Cannot V-Cut a curve or a line that is either not horizontal or vertical.\n"
+                message += "Possible cause might be:\n"
+                message += "- your tabs hit a curved boundary of your PCB,\n"
+                message += "- your vertical or horizontal PCB edges are not precisely vertical or horizontal.\n"
+                message += "Modify the design or accept curve approximation via V-cuts."
+                raise RuntimeError(message)
             cut = cut.simplify(1).parallel_offset(offset, "left")
             start = roundPoint(cut.coords[0])
             end = roundPoint(cut.coords[-1])
@@ -1456,7 +1460,13 @@ class Panel:
             elif start.y == end.y or (abs(start.y - end.y) <= fromMm(0.5) and boundCurves):
                 self.addVCutH((start.y + end.y) / 2)
             else:
-                raise RuntimeError("Cannot perform V-Cut which is not horizontal or vertical")
+                description = f"[{toMm(start.x)}, {toMm(start.y)}] -> [{toMm(end.x)}, {toMm(end.y)}]"
+                message = f"Cannot perform V-Cut which is not horizontal or vertical ({description}).\n"
+                message += "Possible cause might be:\n"
+                message += "- check that intended edges are truly horizonal or vertical\n"
+                message += "- check your tab placement if it as expected\n"
+                message += "You can use layer style of cuts to see them and validate them."
+                raise RuntimeError(message)
 
     def makeMouseBites(self, cuts, diameter, spacing, offset=fromMm(0.25),
         prolongation=fromMm(0.5)):
@@ -1483,7 +1493,8 @@ class Panel:
                     hole = cut.interpolate( i * length / (count - 1) )
                 if bloatedSubstrate.intersects(hole):
                     self.addNPTHole(toKiCADPoint((hole.x, hole.y)), diameter,
-                                    ref=f"KiKit_MB_{self.renderedMousebiteCounter}_{i+1}")
+                                    ref=f"KiKit_MB_{self.renderedMousebiteCounter}_{i+1}",
+                                    excludedFromPos=True)
 
     def makeCutsToLayer(self, cuts, layer=Layer.Cmts_User, prolongation=fromMm(0)):
         """
@@ -1506,7 +1517,10 @@ class Panel:
                 self.board.Add(segment)
 
     def addNPTHole(self, position: VECTOR2I, diameter: KiLength,
-                   paste: bool=False, ref: Optional[str]=None) -> None:
+                   paste: bool=False, ref: Optional[str]=None,
+                   excludedFromPos: bool=False,
+                   solderMaskMargin: Optional[KiLength] = None,
+    ) -> None:
         """
         Add a drilled non-plated hole to the position (`VECTOR2I`) with given
         diameter. The paste option allows to place the hole on the paste layers.
@@ -1516,6 +1530,8 @@ class Panel:
         for pad in footprint.Pads():
             pad.SetDrillSize(toKiCADPoint((diameter, diameter)))
             pad.SetSize(toKiCADPoint((diameter, diameter)))
+            if solderMaskMargin is not None:
+                footprint.SetLocalSolderMaskMargin(solderMaskMargin)
             if paste:
                 layerSet = pad.GetLayerSet()
                 layerSet.AddLayer(Layer.F_Paste)
@@ -1523,6 +1539,10 @@ class Panel:
                 pad.SetLayerSet(layerSet)
         if ref is not None:
             footprint.SetReference(ref)
+        if hasattr(footprint, "SetExcludedFromPosFiles"): # KiCAD 6 doesn't support this attribute
+            footprint.SetExcludedFromPosFiles(excludedFromPos)
+        if hasattr(footprint, "SetBoardOnly"):
+            footprint.SetBoardOnly(True)
         self.board.Add(footprint)
 
     def addFiducial(self, position: VECTOR2I, copperDiameter: KiLength,
@@ -1584,16 +1604,19 @@ class Panel:
                              paste, ref = f"KiKit_FID_B_{i+1}")
 
     def addCornerTooling(self, holeCount, horizontalOffset, verticalOffset,
-                         diameter, paste=False):
+                         diameter, paste=False, solderMaskMargin: Optional[KiLength]=None):
         """
         Add up to 4 tooling holes to the top-left, top-right, bottom-left and
         bottom-right corner of the board (in this order). This function expects
         there is enough space on the board/frame/rail to place the feature.
 
         The offsets are measured from the outer edges of the substrate.
+
+        Optionally, a solder mask margin (diameter) can also be specified.
         """
         for i, pos in enumerate(self.panelCorners(horizontalOffset, verticalOffset)[:holeCount]):
-            self.addNPTHole(pos, diameter, paste, ref=f"KiKit_TO_{i+1}")
+            self.addNPTHole(pos, diameter, paste, ref=f"KiKit_TO_{i+1}", excludedFromPos=False,
+                            solderMaskMargin=solderMaskMargin)
 
     def addMillFillets(self, millRadius):
         """
@@ -1757,7 +1780,7 @@ class Panel:
         faceDistances = [(x, np.around(partitionFace.distance(x), 2)) for x in faceSegments]
         minFaceDistance = min(faceDistances, key=lambda x: x[1])[1]
 
-        cutFaces = [x for x, d in faceDistances if d == minFaceDistance]
+        cutFaces = [x for x, d in faceDistances if abs(d - minFaceDistance) < SHP_EPSILON]
         tabs, cuts = [], []
         for cutLine in listGeometries(shapely.ops.unary_union(cutFaces)):
             if minFaceDistance > 0: # Do not generate degenerated polygons
@@ -1807,6 +1830,10 @@ class Panel:
 
         return cuts
 
+    def inheritLayerNames(self, board):
+        for layer in pcbnew.LSET.AllLayersMask().Seq():
+            name = board.GetLayerName(layer)
+            self.board.SetLayerName(layer, name)
 
     def inheritCopperLayers(self, board):
         """
@@ -1832,6 +1859,8 @@ class Panel:
             strokeWidth: KiLength=fromMm(1), strokeSpacing: KiLength=fromMm(1),
             orientation: KiAngle=fromDegrees(45)) -> None:
         """
+        This function is deprecated, please, use panel features instead.
+
         Fill given layers with copper on unused areas of the panel (frame, rails
         and tabs). You can specify the clearance, if it should be hatched
         (default is solid) or shape the strokes of hatched pattern.
@@ -2221,6 +2250,12 @@ class Panel:
             vDim.SetExtensionOffset(-self.filletSize)
         self.board.Add(vDim)
 
+    def apply(self, feature: Any) -> None:
+        """
+        Apply given feature to the panel
+        """
+        feature.apply(self)
+
 
 def getFootprintByReference(board, reference):
     """
